# -- BlockMania --
# --  mei 2023  --

import pygame

import blockmania_display
import blockmania_controller
import blockmania_data
from general_states.screen_01_title import screen_title
from general_states.screen_02_play import score_manager
from general_states.screen_03_preferences import screen_preferences
from general_states.screen_04_help import screen_help
from util.block_tools import color_tool
from view import field
from util import font_manager

fps = 30


def main():
    start_gameloop()


def init():

    color_tool.init()
    font_manager.init()
    score_manager.init()

    screen_title.init()
    screen_help.init()
    screen_preferences.init()

    field.init()
    blockmania_controller.init()


def start_gameloop():
    clock = pygame.time.Clock()

    while True:
        events = pygame.event.get()
        for e in events:

            if e.type == pygame.QUIT:  # Handle the 'close'-icon.
                return

            if e.type == pygame.WINDOWSHOWN:
                init()

            # if e.type == pygame.MOUSEBUTTONDOWN:
            #     mouse_presses = pygame.mouse.get_pressed()
            #     pos = pygame.mouse.get_pos()
            #     if mouse_presses[0]:
            #         blockmania_controller.handle_mouse_click_left(pos)
            #     elif mouse_presses[2]:
            #         blockmania_controller.handle_mouse_click_right(pos)

            if e.type == pygame.KEYDOWN:
                match e.key:
                    case pygame.K_F1     : blockmania_controller.set_user_input(blockmania_data.EnumUserInput.PLAY_SCREEN)
                    case pygame.K_F2     : blockmania_controller.set_user_input(blockmania_data.EnumUserInput.TITLE_SCREEN)
                    case pygame.K_F3     : blockmania_controller.set_user_input(blockmania_data.EnumUserInput.HELP_SCREEN)
                    case pygame.K_F4     : blockmania_controller.set_user_input(blockmania_data.EnumUserInput.PREFS_SCREEN)

                    case pygame.K_F5     : blockmania_controller.set_user_input(blockmania_data.EnumUserInput.F5)
                    case pygame.K_F6     : blockmania_controller.set_user_input(blockmania_data.EnumUserInput.F6)
                    case pygame.K_F7     : blockmania_controller.set_user_input(blockmania_data.EnumUserInput.F7)
                    case pygame.K_F8     : blockmania_controller.set_user_input(blockmania_data.EnumUserInput.F8)

                    case pygame.K_SPACE  : blockmania_controller.set_user_input(blockmania_data.EnumUserInput.FLIP)
                    case pygame.K_r      : blockmania_controller.set_user_input(blockmania_data.EnumUserInput.ROTATE)

                    case pygame.K_UP     : blockmania_controller.set_user_input(blockmania_data.EnumUserInput.ROTATE)
                    case pygame.K_DOWN   : blockmania_controller.set_user_input(blockmania_data.EnumUserInput.MOVE_DOWN)
                    case pygame.K_LEFT   : blockmania_controller.set_user_input(blockmania_data.EnumUserInput.MOVE_LEFT)
                    case pygame.K_RIGHT  : blockmania_controller.set_user_input(blockmania_data.EnumUserInput.MOVE_RIGHT)

                    case pygame.K_w      : blockmania_controller.set_user_input(blockmania_data.EnumUserInput.ROTATE)
                    case pygame.K_s      : blockmania_controller.set_user_input(blockmania_data.EnumUserInput.MOVE_DOWN)
                    case pygame.K_a      : blockmania_controller.set_user_input(blockmania_data.EnumUserInput.MOVE_LEFT)
                    case pygame.K_d      : blockmania_controller.set_user_input(blockmania_data.EnumUserInput.MOVE_RIGHT)

                    case pygame.K_t      : blockmania_controller.set_user_input(blockmania_data.EnumUserInput.SELECT_BLOCK)
                    case pygame.K_ESCAPE : blockmania_controller.set_user_input(blockmania_data.EnumUserInput.ESCAPE)

            elif e.type == pygame.KEYUP:
                blockmania_controller.set_user_input(blockmania_data.EnumUserInput.NONE)

        blockmania_display.handle_gametick()
        blockmania_controller.handle_gametick()

        pygame.display.update()
        clock.tick(fps)


if __name__ == '__main__':
    main()
