from enum import Enum

import blockmania_data
from blockmania_data import EnumUserInput
from general_states.screen_01_title import screen_title
from general_states.screen_02_play import controller_play_screen
from general_states.screen_03_preferences import screen_preferences
from general_states.screen_04_help import screen_help
from util import tools
from util.classes.class_RepetitionChecker import RepetitionChecker
from view import preview, field

repetitionChecker = RepetitionChecker()


class EnumState(Enum):
    UNDEFINED    = 10
    TITLE_SCREEN = 20
    PLAY_SCREEN  = 30
    PREFS_SCREEN = 40
    HELP_SCREEN  = 50


class gb:  # global data.
    state: int = EnumState.UNDEFINED
    user_input: EnumUserInput = blockmania_data.EnumUserInput.NONE


def init():

    preview.set_random_block()

    gb.state = EnumState.TITLE_SCREEN
    controller_play_screen.init()


def handle_gametick():

    user_input = gb.user_input
    is_input_repeating = repetitionChecker.is_repeating(user_input)

    if user_input != blockmania_data.EnumUserInput.NONE:
        handle_input(user_input, is_input_repeating)

    distribute_gametick(is_input_repeating, user_input)


def distribute_gametick(is_input_repeating, user_input):

    match gb.state:
        case EnumState.TITLE_SCREEN : screen_title.handle_gametick()
        case EnumState.PLAY_SCREEN  : controller_play_screen.handle_gametick(user_input, is_input_repeating)
        case EnumState.HELP_SCREEN  : screen_help.handle_gametick()
        case EnumState.PREFS_SCREEN : screen_preferences.handle_gametick()


def handle_input(user_input: EnumUserInput, is_input_repeating):

    if not is_input_repeating and gb.state != EnumState.PLAY_SCREEN:

        match user_input:
            case blockmania_data.EnumUserInput.TITLE_SCREEN : switch_to_title_screen()
            case blockmania_data.EnumUserInput.PREFS_SCREEN : switch_to_preferences_screen()
            case blockmania_data.EnumUserInput.HELP_SCREEN  : switch_to_help_screen()


def switch_to_play_screen():
    controller_play_screen.reset()
    gb.state = EnumState.PLAY_SCREEN


def switch_to_title_screen():
    screen_title.reset()
    gb.state = EnumState.TITLE_SCREEN


def switch_to_preferences_screen():
    gb.state = EnumState.PREFS_SCREEN


def switch_to_help_screen():
    gb.state = EnumState.HELP_SCREEN


def set_user_input(input_new: EnumUserInput):
    gb.user_input = input_new


def handle_mouse_click_left(pos: tuple[int, int]):
    tile = tools.get_tile_from_mouse_click(pos)
    field.add_tile(tile, 6)


def handle_mouse_click_right(pos: tuple[int, int]):
    tile = tools.get_tile_from_mouse_click(pos)
    field.remove_tile(tile)
