import pygame
from pygame.surface import Surface
from enum import Enum

import blockmania_controller
import blockmania_data
import blockmania_display
from general_states.screen_02_play import transition
from util import font_manager, tools
from util.classes.class_BlinkingText import BlinkingText
from util.classes.class_RepetitionChecker import RepetitionChecker

repetitionChecker = RepetitionChecker()


class EnumScreenState(Enum):
    NORMAL     = 10
    TRANSITION = 20


class gb:  # global data.

    state: EnumScreenState = EnumScreenState.NORMAL

    text_01_surface: Surface
    text_01_x = 0

    blinking_text: BlinkingText

    text_03_surface: Surface
    text_03_x = 0
    text_03_y = 0

    text_04_surface: Surface
    text_04_x = 0
    text_04_y = 0


def init():

    gb.text_01_surface = font_manager.gb.font_01.render("BlockMania", True, pygame.Color('red'))
    gb.text_01_x = tools.calculate_x_for_text(gb.text_01_surface)

    text_02_surface: Surface = font_manager.gb.font_02.render("Press 'F1' to play", True, pygame.Color('white'))
    text_02_x = tools.calculate_x_for_text(text_02_surface)
    gb.blinking_text = BlinkingText(text_02_surface, text_02_x, 175)

    gb.text_03_surface = font_manager.gb.font_04.render("How to play : F3", True, pygame.Color('orange'))
    gb.text_03_x = tools.calculate_x_for_text(gb.text_03_surface)
    gb.text_03_y = blockmania_data.WINDOW_SIZE[1] - 120

    gb.text_04_surface = font_manager.gb.font_04.render("Preferences : F4", True, pygame.Color('orange'))
    gb.text_04_x = tools.calculate_x_for_text(gb.text_04_surface)
    gb.text_04_y = blockmania_data.WINDOW_SIZE[1] - 80


def handle_gametick():

    draw()

    match gb.state:
        case EnumScreenState.NORMAL     : handle_input()
        case EnumScreenState.TRANSITION : transition.trigger_part_one()


def reset():
    gb.blinking_text.reset()


def handle_input():

    if blockmania_controller.gb.user_input == blockmania_data.EnumUserInput.NONE:
        return

    if repetitionChecker.is_repeating(blockmania_controller.gb.user_input):
        return

    if blockmania_controller.gb.user_input == blockmania_data.EnumUserInput.PLAY_SCREEN:
        gb.state = EnumScreenState.TRANSITION


def draw():
    blockmania_display.canvas.blit(gb.text_01_surface, (gb.text_01_x, 40))
    gb.blinking_text.handle_gametick()
    blockmania_display.canvas.blit(gb.text_03_surface, (gb.text_03_x, gb.text_03_y))
    blockmania_display.canvas.blit(gb.text_04_surface, (gb.text_04_x, gb.text_04_y))
