from enum import Enum

import pygame

import blockmania_data
from blockmania_data import EnumUserInput
from general_states.screen_02_play import state_01_regular, state_02_row_removal, score_manager, state_03_paused, state_04_game_over
from util import font_manager
from util.classes.class_BlinkingText import BlinkingText
from util.classes.class_RepetitionChecker import RepetitionChecker
from view import preview, field

repetitionChecker = RepetitionChecker()


class EnumDirection(Enum):
    NONE  = 10
    UP    = 20
    DOWN  = 30
    LEFT  = 40
    RIGHT = 50


class EnumState(Enum):
    UNDEFINED   = 10
    REGULAR     = 20
    ROW_REMOVAL = 30
    PAUSED      = 40
    GAME_OVER   = 50


class gb:  # global data.

    state = EnumState.UNDEFINED
    state_previous = EnumState.UNDEFINED

    offset_col: int = 0
    offset_row: int = 0

    blinking_text_paused: BlinkingText
    blinking_text_game_over: BlinkingText

    paused: bool = False


def init():

    init_texts()
    gb.state = EnumState.REGULAR

    state_03_paused.init()
    state_04_game_over.init()


def init_texts():

    surface = font_manager.gb.font_02.render("paused", True, pygame.Color('white'))
    gb.blinking_text_paused = BlinkingText(surface, 165, 225)

    surface = font_manager.gb.font_02.render("game over", True, pygame.Color('white'))
    gb.blinking_text_game_over = BlinkingText(surface, 145, 225)


def reset():
    field.reset()
    score_manager.reset()
    gb.state = EnumState.REGULAR


def handle_gametick(user_input: EnumUserInput, is_input_repeating: bool):

    preview.render()
    field.render()
    score_manager.render()

    distribute_gametick(is_input_repeating, user_input)

    handle_input(user_input, is_input_repeating)


def distribute_gametick(is_input_repeating, user_input):

    match gb.state:
        case EnumState.REGULAR     : state_01_regular.handle_gametick()
        case EnumState.ROW_REMOVAL : state_02_row_removal.handle_gametick()
        case EnumState.PAUSED      : state_03_paused.handle_gametick()
        case EnumState.GAME_OVER   : state_04_game_over.handle_gametick(user_input, is_input_repeating)


def handle_input(user_input: EnumUserInput, is_input_repeating):

    if not is_input_repeating:

        match user_input:
            case blockmania_data.EnumUserInput.ESCAPE : toggle_pause_if_appropriate()


def switch_state(new_state: EnumState):

    gb.state_previous = gb.state

    match new_state :
        case EnumState.REGULAR     : switch_to_regular_game_play()
        case EnumState.ROW_REMOVAL : switch_to_row_removal()
        case EnumState.PAUSED      : switch_to_paused()
        case EnumState.GAME_OVER   : switch_to_game_over()


def switch_to_regular_game_play():
    state_01_regular.gb.state = state_01_regular.EnumScreenState.TRANSITION
    gb.state = EnumState.REGULAR


def switch_to_row_removal():
    state_02_row_removal.init()
    gb.state = EnumState.ROW_REMOVAL


def switch_to_paused():
    state_03_paused.reset()
    gb.state = EnumState.PAUSED


def switch_to_game_over():
    state_04_game_over.reset()
    gb.state = EnumState.GAME_OVER


def switch_back_to_previous_state():
    switch_state(gb.state_previous)


def toggle_pause_if_appropriate():

    if gb.state == EnumState.REGULAR or gb.state == EnumState.PAUSED:

        gb.paused = not gb.paused
        if gb.paused:
            switch_state(EnumState.PAUSED)
        else:
            switch_state(gb.state_previous)

