from enum import Enum

import blockmania_controller
import blockmania_data
from general_states.screen_02_play import controller_play_screen as controller, block_speed, controller_play_screen, transition
from util.block_tools import copy_tool, tile_tool, block_tool, offset_tool, location_check_tool, block_rotate_tool, block_flip_tool, row_check_tool
from util.classes.class_Delay import Delay
from util.classes.class_RepetitionChecker import RepetitionChecker
from view import field, preview

repetitionChecker = RepetitionChecker()
delay_key_repeat = Delay(100)


class EnumScreenState(Enum):
    TRANSITION = 10
    NORMAL     = 20


class gb:  # global data.
    state: EnumScreenState = EnumScreenState.TRANSITION
    dummy_block = block_tool.create_block(0)


def handle_gametick():

    match gb.state:
        case EnumScreenState.TRANSITION : transition.trigger_part_two()
        case EnumScreenState.NORMAL     : main_loop()


def main_loop():
    if field.is_er_een_block_op_het_veld():

        if is_het_tijd_voor_een_move_down():
            move_block_down()
        else:
            handle_user_input()

    else:  # Er is nog [geen] block op het veld :
        move_preview_block_to_field()


def is_het_tijd_voor_een_move_down():
    return block_speed.gb.delay.has_expired()


def move_block_down():

    copy_tool.copy(field.get_block(), gb.dummy_block)
    gb.dummy_block.increment_row_offset()
    tile_tool.refresh_tiles(gb.dummy_block)

    if location_check_tool.vallen_alle_block_tile_locaties_binnen_de_field_grens(gb.dummy_block) and \
            location_check_tool.zijn_alle_block_tile_locaties_vrij_op_het_field(gb.dummy_block):
        copy_tool.copy(gb.dummy_block, field.get_block())
    else:
        field.make_block_permanent()
        if row_check_tool.bevat_het_field_volledig_gevulde_regels():
            controller.switch_to_row_removal()


def handle_user_input():

    is_not_repeating = repetitionChecker.is_not_repeating(blockmania_controller.gb.user_input)

    if blockmania_controller.gb.user_input == blockmania_data.EnumUserInput.SELECT_BLOCK and is_not_repeating:
        preview.set_next_block()

    elif (blockmania_controller.gb.user_input == blockmania_data.EnumUserInput.ROTATE or
            blockmania_controller.gb.user_input == blockmania_data.EnumUserInput.FLIP) and is_not_repeating:
        move_or_rotate_block()

    else:

        if is_not_repeating:
            handle_move()
            delay_key_repeat.reset()
        else:
            if delay_key_repeat.has_expired():
                handle_move()


def handle_move():

    match blockmania_controller.gb.user_input :
        case blockmania_data.EnumUserInput.MOVE_DOWN  : move_block_down()
        case blockmania_data.EnumUserInput.MOVE_UP    : move_or_rotate_block()
        case blockmania_data.EnumUserInput.MOVE_LEFT  : move_or_rotate_block()
        case blockmania_data.EnumUserInput.MOVE_RIGHT : move_or_rotate_block()


def move_preview_block_to_field():

    copy_tool.copy(preview.get_block(), gb.dummy_block)

    offset_tool.calculate_initial_col_offset(gb.dummy_block)
    tile_tool.refresh_tiles(gb.dummy_block)

    if location_check_tool.zijn_alle_block_tile_locaties_vrij_op_het_field(gb.dummy_block):

        block_new = block_tool.create_block(0)
        copy_tool.copy(gb.dummy_block, block_new)

        block_speed.gb.delay.reset()
        field.set_block(block_new)

        # preview.set_block_by_number(0)  # Block 'O'
        # preview.set_block_by_number(6)  # Block 'I'
        preview.set_random_block()

    else:
        game_over()


def game_over():
    controller_play_screen.switch_to_game_over()


def move_or_rotate_block():

    copy_tool.copy(field.get_block(), gb.dummy_block)

    match blockmania_controller.gb.user_input:
        case blockmania_data.EnumUserInput.MOVE_UP:    gb.dummy_block.decrement_row_offset()
        case blockmania_data.EnumUserInput.MOVE_DOWN:  gb.dummy_block.increment_row_offset()
        case blockmania_data.EnumUserInput.MOVE_LEFT:  gb.dummy_block.decrement_col_offset()
        case blockmania_data.EnumUserInput.MOVE_RIGHT: gb.dummy_block.increment_col_offset()
        case blockmania_data.EnumUserInput.ROTATE:     block_rotate_tool.rotate(gb.dummy_block)
        case blockmania_data.EnumUserInput.FLIP:       block_flip_tool.flip(gb.dummy_block)

    tile_tool.refresh_tiles(gb.dummy_block)

    if location_check_tool.vallen_alle_block_tile_locaties_binnen_de_field_grens(gb.dummy_block) and \
            location_check_tool.zijn_alle_block_tile_locaties_vrij_op_het_field(gb.dummy_block):
        copy_tool.copy(gb.dummy_block, field.get_block())
