from enum import Enum

from general_states.screen_02_play import row_marker, row_remover, controller_play_screen
from general_states.screen_02_play.row_marker import EnumState, EnumActionType
from util.block_tools import row_check_tool
from view import field


class EnumRemovalState(Enum):
    MARK_TILES   = 10
    REMOVE_TILES = 20
    REMOVE_ROWS  = 30


class gb:  # global data.
    state = EnumRemovalState.MARK_TILES
    completed_rows  = []


def init():

    gb.completed_rows = row_check_tool.gb.completed_rows
    row_marker.start(gb.completed_rows, EnumActionType.MARK)
    gb.state = EnumRemovalState.MARK_TILES


def handle_gametick():

    match gb.state:
        case EnumRemovalState.MARK_TILES   : mark_tiles()
        case EnumRemovalState.REMOVE_TILES : remove_tiles()
        case EnumRemovalState.REMOVE_ROWS  : remove_rows()


def mark_tiles():

    if row_marker.gb.state == EnumState.IN_PROGRESS:
        row_marker.handle_gametick()
    else:
        row_marker.start(gb.completed_rows, EnumActionType.REMOVE)  # Switch the internal state of the row-marker.
        gb.state = EnumRemovalState.REMOVE_TILES


def remove_tiles():

    if row_marker.gb.state == EnumState.IN_PROGRESS:
        row_marker.handle_gametick()
    else:
        gb.state = EnumRemovalState.REMOVE_ROWS


def remove_rows():

    row_remover.remove_rows(gb.completed_rows)

    gb.state = EnumRemovalState.MARK_TILES
    controller_play_screen.switch_to_regular_game_play()


def add_test_data():

    # r: int = 0
    # coords: list[tuple[int, int]] = [(r,0),(r,1),(r,8),(r,9)]
    # field.create_tiles(coords, 1)

    r: int = 16
    coords: list[tuple[int, int]] = [(r,0),(r,1),(r,4),(r,5),(r,6),(r,8),(r,9)]
    field.create_tiles(coords, 1)

    r: int = 17
    coords: list[tuple[int, int]] = [(r,0),(r,1),(r,2),(r,3),(r,4),(r,5),(r,6),(r,7),(r,8),(r,9)]
    field.create_tiles(coords, 2)

    r: int = 18
    coords: list[tuple[int, int]] = [(r,3),(r,4),(r,5)]
    field.create_tiles(coords, 3)

    r: int = 19
    coords: list[tuple[int, int]] = [(r,0),(r,1),(r,2),(r,3),(r,4),(r,5),(r,6),(r,7),(r,8),(r,9)]
    field.create_tiles(coords, 4)



    # r: int = 12
    # coords: list[tuple[int, int]] = [(r,0),(r,1),(r,4),(r,5),(r,6),(r,8),(r,9)]
    # field.create_tiles(coords, 1)
    #
    # r: int = 13
    # coords: list[tuple[int, int]] = [(r,0),(r,1),(r,2),(r,3),(r,4),(r,5),(r,6),(r,7),(r,8),(r,9)]
    # field.create_tiles(coords, 2)
    #
    # r: int = 14
    # coords: list[tuple[int, int]] = [(r,3),(r,4),(r,5)]
    # field.create_tiles(coords, 3)
    #
    # r: int = 15
    # coords: list[tuple[int, int]] = [(r,0),(r,1),(r,2),(r,3),(r,4),(r,5),(r,6),(r,7),(r,8),(r,9)]
    # field.create_tiles(coords, 4)
    #
    #
    #
    #
    # r: int = 8
    # coords: list[tuple[int, int]] = [(r,0),(r,1),(r,4),(r,5),(r,6),(r,8),(r,9)]
    # field.create_tiles(coords, 1)
    #
    # r: int = 9
    # coords: list[tuple[int, int]] = [(r,0),(r,1),(r,2),(r,3),(r,4),(r,5),(r,6),(r,7),(r,8),(r,9)]
    # field.create_tiles(coords, 2)
    #
    # r: int = 10
    # coords: list[tuple[int, int]] = [(r,3),(r,4),(r,5)]
    # field.create_tiles(coords, 3)
    #
    # r: int = 11
    # coords: list[tuple[int, int]] = [(r,0),(r,1),(r,2),(r,3),(r,4),(r,5),(r,6),(r,7),(r,8),(r,9)]
    # field.create_tiles(coords, 4)
