import pygame
import blockmania_display
import blockmania_data
from pygame.surface import Surface

from util.block_tools import color_tool
from util.classes.class_Tile import Tile


def get_tile_y(row) -> int:
    return (row * blockmania_data.TILE_HEIGHT + blockmania_data.GAME_FIELD_LOCATION_Y)


def calculate_x_for_text(text_surface: Surface) -> int:
    return (blockmania_data.WINDOW_SIZE[0] - text_surface.get_width()) // 2


def calculate_x_for_game_field_text(text_surface: Surface) -> int:
    return ((blockmania_data.GAME_FIELD_WIDTH - text_surface.get_width()) // 2) + blockmania_data.GAME_FIELD_LOCATION_X


def draw_rectangle_with_border_and_fill(x: int, y: int, width: int, height: int, color: pygame.Color, border_width: int, border_color: pygame.Color):
    """
    Draws a rectangle with a border and a fill.
    The border is on the [outside] of the fill,
    so x, y, width and height are all related to the fill.
    """
    pygame.draw.rect(blockmania_display.canvas, border_color, pygame.Rect(  # Draw border.
        (x - border_width),
        (y - border_width),
        (width + border_width * 2),
        (height + border_width * 2)))

    pygame.draw.rect(blockmania_display.canvas, color, pygame.Rect(
        x, y, width, height))  # Draw fill.


def copy_2d_array(array_source: list[list[int]]) -> list[list[int]]:
    """Deze functie maakt een copy van een 'vierkante' 2d-array"""

    array_copy = []
    array_size = len(array_source[0])

    for row in range(array_size):
        row_list = []
        for col in range(array_size):
            row_list.append(array_source[row][col])
        array_copy.append(row_list)

    return array_copy


def get_tile_from_mouse_click(pos: tuple[int, int]) -> Tile:
    row = int((pos[1] - blockmania_data.GAME_FIELD_LOCATION_Y) / blockmania_data.TILE_HEIGHT)
    col = int((pos[0] - blockmania_data.GAME_FIELD_LOCATION_X) / blockmania_data.TILE_WIDTH)
    return Tile(row, col)


def create_2d_array(rows: int, cols: int) -> []:
    table = []
    for r in range(rows):
        row_list = []
        for c in range(cols):
            row_list.append(0)
        table.append(row_list)
    return table


def draw_tile(x, y, width, height, color_code):
    color_number = color_code if (color_code < 100) else (color_code - 100)
    draw_removal_mark = (color_code > 100)

    x_top_left = x
    y_top_left = y

    x_bottom_right = x_top_left + width  - 1
    y_bottom_right = y_top_left + height - 1

    COLOR_TINT  = 0
    COLOR_BASE  = 1
    COLOR_SHADE = 2

    # Draw body :

    pygame.draw.rect(
        blockmania_display.canvas, color_tool.gb.tile_colors[color_number][COLOR_BASE],
        pygame.Rect(
            x_top_left,
            y_top_left,
            width,
            height))

    # Draw left & top border :

    pygame.draw.line(blockmania_display.canvas, color_tool.gb.tile_colors[color_number][COLOR_TINT],
                     (x_top_left, y_top_left), (x_top_left + width - 1, y_top_left))

    pygame.draw.line(blockmania_display.canvas, color_tool.gb.tile_colors[color_number][COLOR_TINT],
                     (x_top_left, y_top_left), (x_top_left, y_top_left + height - 1))

    x_top_left += 1
    y_top_left += 1

    pygame.draw.line(blockmania_display.canvas, color_tool.gb.tile_colors[color_number][COLOR_TINT],
                     (x_top_left, y_top_left), (x_top_left + width - 3, y_top_left))

    pygame.draw.line(blockmania_display.canvas, color_tool.gb.tile_colors[color_number][COLOR_TINT],
                     (x_top_left, y_top_left), (x_top_left, y_top_left + height - 3))

    # Draw right & bottom border :

    pygame.draw.line(blockmania_display.canvas, color_tool.gb.tile_colors[color_number][COLOR_SHADE],
                     (x_bottom_right, y_bottom_right), (x_bottom_right, y_bottom_right - height + 2))

    pygame.draw.line(blockmania_display.canvas, color_tool.gb.tile_colors[color_number][COLOR_SHADE],
                     (x_bottom_right, y_bottom_right), (x_bottom_right - width + 2, y_bottom_right))

    x_bottom_right -= 1
    y_bottom_right -= 1

    pygame.draw.line(blockmania_display.canvas, color_tool.gb.tile_colors[color_number][COLOR_SHADE],
                     (x_bottom_right, y_bottom_right), (x_bottom_right, y_bottom_right - height + 4))

    pygame.draw.line(blockmania_display.canvas, color_tool.gb.tile_colors[color_number][COLOR_SHADE],
                     (x_bottom_right, y_bottom_right), (x_bottom_right - width + 4, y_bottom_right))

    # Draw removal mark, if appropriate :

    if draw_removal_mark:
        draw_rectangle_with_border_and_fill(x + 6, y + 6,
                                            width - 12, height - 12,
                                            color_tool.gb.tile_colors[color_number][COLOR_BASE], 3,
                                            color_tool.gb.tile_colors[0][0])
