import pygame

import blockmania_data
import blockmania_display
from util import tools
from util.classes.class_Block import Block
from util.classes.class_Tile import Tile

grid_horizontal_start_points = []
grid_horizontal_end_points   = []

grid_vertical_start_points = []
grid_vertical_end_points   = []


class gb:  # global data.
    block: Block = None
    grid = []


def init():
    calculate_grid_coordinates()
    reset()


def reset():
    gb.grid = [[0 for c in range(blockmania_data.COL_COUNT)] for r in range(blockmania_data.ROW_COUNT)]


def set_block(block):
    gb.block = block


def get_block():
    return gb.block


def is_er_een_block_op_het_veld():
    return (gb.block is not None)


def make_block_permanent():
    for tile in gb.block.tiles:
        add_tile(tile, gb.block.color_code)
    gb.block = None


def is_location_occupied(row, col):
    return (gb.grid[row][col] != 0)


def calculate_grid_coordinates():

    for row in range(blockmania_data.ROW_COUNT):

        grid_horizontal_start_points.append(
            (blockmania_data.GAME_FIELD_LOCATION_X,
             blockmania_data.GAME_FIELD_LOCATION_Y + (row * blockmania_data.TILE_HEIGHT)) )

        grid_horizontal_end_points.append(
            (blockmania_data.GAME_FIELD_LOCATION_X + blockmania_data.GAME_FIELD_WIDTH,
             blockmania_data.GAME_FIELD_LOCATION_Y + (row * blockmania_data.TILE_HEIGHT)))

    for col in range(blockmania_data.COL_COUNT):

        grid_vertical_start_points.append(
            (blockmania_data.GAME_FIELD_LOCATION_X + (col * blockmania_data.TILE_WIDTH),
             blockmania_data.GAME_FIELD_LOCATION_Y) )

        grid_vertical_end_points.append(
            (blockmania_data.GAME_FIELD_LOCATION_X + (col * blockmania_data.TILE_WIDTH),
             blockmania_data.GAME_FIELD_LOCATION_Y + blockmania_data.GAME_FIELD_HEIGHT) )


def render():
    draw_game_field()
    draw_grid()
    draw_permanent_tiles()
    draw_block_tiles()


def draw_game_field():

    tools.draw_rectangle_with_border_and_fill(
        blockmania_data.GAME_FIELD_LOCATION_X, blockmania_data.GAME_FIELD_LOCATION_Y,
        blockmania_data.GAME_FIELD_WIDTH, blockmania_data.GAME_FIELD_HEIGHT,
        blockmania_data.GAME_FIELD_BACKGROUND_COLOR,
        blockmania_data.GAME_FIELD_BORDER_WIDTH, blockmania_data.GAME_FIELD_BORDER_COLOR)


def draw_grid():

    for row in range(1, blockmania_data.ROW_COUNT):
        pygame.draw.line(
            blockmania_display.canvas, blockmania_data.GAME_FIELD_GRID_COLOR,
            grid_horizontal_start_points[row],
            grid_horizontal_end_points[row], 1)

    for col in range(1, blockmania_data.COL_COUNT):
        pygame.draw.line(
            blockmania_display.canvas, blockmania_data.GAME_FIELD_GRID_COLOR,
            grid_vertical_start_points[col],
            grid_vertical_end_points[col], 1)


def draw_permanent_tiles():

    for row in range(blockmania_data.ROW_COUNT):
        for col in range(blockmania_data.COL_COUNT):
            color_code = gb.grid[row][col]
            if color_code > 0:
                draw_tile(col, row, color_code)


def draw_tile(col, row, color_code):

    x = (col * blockmania_data.TILE_WIDTH)  + blockmania_data.GAME_FIELD_LOCATION_X
    y = (row * blockmania_data.TILE_HEIGHT) + blockmania_data.GAME_FIELD_LOCATION_Y

    width  = blockmania_data.TILE_WIDTH
    height = blockmania_data.TILE_HEIGHT

    tools.draw_tile(x, y, width, height, color_code)


def draw_block_tiles():
    if gb.block is not None:
        for tile in gb.block.tiles:
            draw_block_tile(tile, gb.block.color_code)


def draw_block_tile(tile, color_code):
    draw_tile(tile.col, tile.row, color_code)


def create_tiles(coords: list[tuple[int, int]], color_code):
    for coord in coords:
        add_tile(Tile(coord[0], coord[1]), color_code)


def add_tile(tile, color_code):
    gb.grid[tile.row][tile.col] = color_code


def remove_tile(tile):
    gb.grid[tile.row][tile.col] = 0


def remove_tile_by_coords(row, col):
    gb.grid[row][col] = 0
